import { Component, OnInit, OnChanges, Input, Output, EventEmitter, ViewChild, AfterViewInit, OnDestroy } from '@angular/core';
import { Router } from '@angular/router';
import { Store, select } from '@ngrx/store';
import { RPCClientError } from '@loafer/ng-rpc/protocol';

import * as ListStore from 'packages/meta/sensor-display-item/store/list';
import { ReadAllSensorDisplayItemByCrawlerSelector } from 'packages/meta/sensor-display-item/store';
import { MetaSensorDisplayItem } from 'packages/meta/sensor-display-item/model/MetaSensorDisplayItem';
import { MetaCrawler } from '../../../../meta/crawler/model/MetaCrawler';
import { ITreeOptions } from 'angular-tree-component';

@Component({
  selector: 'of-sensor-item-selector',
  templateUrl: './sensor-item-selector.component.html',
  styleUrls: ['./sensor-item-selector.component.scss']
})
export class SensorItemSelectorComponent implements OnInit, OnChanges {

  @Input() selectedCrawler: MetaCrawler;
  @Output() itemSelectEvent = new EventEmitter<Set<MetaSensorDisplayItem>>();
  items$ = this.listStore.pipe(select(ReadAllSensorDisplayItemByCrawlerSelector.select('list')));
  nodes;
  checkedSet: Set<MetaSensorDisplayItem> = new Set();

  constructor(
    private router: Router,
    private listStore: Store<ListStore.State>,
  ) { }

  ngOnInit() {
    this.items$.subscribe(
      (list: MetaSensorDisplayItem[]) => {
        if (list !== null) {
          this.generateTreeNode(list);
        }
      },
      (error: RPCClientError) => {
        console.log(error.response.message);
      }
    );
  }

  ngOnChanges() {
    this.checkedSet.clear();
    this.listStore.dispatch(new ListStore.ReadAllByCrawler(this.selectedCrawler));
  }

  generateTreeNode(list: MetaSensorDisplayItem[]) {
    this.nodes = new Array();
    for (const item of list) {
      const categoryNode = this.existCategory(item);
      if (categoryNode === null) {
        const childrenNode = new Array();
        childrenNode.push(this.getChildNode(item));
        const node = {
          title: item.itemType.name,
          children: childrenNode,
        };
        this.nodes.push(node);
      } else {
        categoryNode.children.push(this.getChildNode(item));
      }
    }
    this.itemSelectEvent.emit(this.checkedSet);
  }

  getChildNode(item) {
    const childNode = {
      title: item.displayName,
      key: ' - ' + item.key,
      default: item.default,
      displayItem: item
    };
    return childNode;
  }

  existCategory(item: MetaSensorDisplayItem) {
    let categoryNode = null;
    for (const node of this.nodes) {
      if (node.title === item.itemType.name) {
        categoryNode = node;
      }
    }
    return categoryNode;
  }

  checkItem(node) {
    node.toggleActivated(true);
    if (node.isActive) {
      if (this.checkedSet.has(node.data.displayItem) === false) {
        this.checkedSet.add(node.data.displayItem);
      }
    } else {
      if (this.checkedSet.has(node.data.displayItem)) {
        this.checkedSet.delete(node.data.displayItem);
      }
    }
    this.itemSelectEvent.emit(this.checkedSet);
  }

  onTreeDataLoad(tree) {
    tree.treeModel.expandAll();
  }
}
